NAME
Minion::Command::minion::schedule - Minion schedule command
SYNOPSIS
Usage: APPLICATION minion schedule [OPTIONS] [NAME]
./myapp.pl minion schedule
./myapp.pl minion schedule daily
./myapp.pl minion schedule -e daily -c '0 4 * * *' -t cleanup
./myapp.pl minion schedule -e foo -c '*/5 * * * *' -t bar -a '[1, 2, 3]' -p 5 -q important
./myapp.pl minion schedule -e weekday_report -c '0 9 * * 1-5' -t report -A 3
./myapp.pl minion schedule -P daily
./myapp.pl minion schedule -r daily
./myapp.pl minion schedule -R daily
./myapp.pl minion schedule -d
Options:
-A, --attempts <number> Number of times each enqueued job will be
attempted, defaults to 1
-a, --args <JSON array> Arguments for the new schedule in JSON format
-c, --cron <expression> Cron expression for new schedule
-d, --dispatch Manually dispatch any due schedules
-E, --expire <seconds> Each enqueued job is valid for this many seconds
before it expires
-e, --enqueue <name> Add or replace a schedule with this name
(requires -c and -t)
-h, --help Show this summary of available options
--home <path> Path to home directory of your application,
defaults to the value of MOJO_HOME or
auto-detection
-l, --limit <number> Number of schedules to show when listing them,
defaults to 100
-m, --mode <name> Operating mode for your application, defaults to
the value of MOJO_MODE/PLACK_ENV or
"development"
-n, --notes <JSON> Notes in JSON format for the new schedule
-o, --offset <number> Number of schedules to skip when listing them,
defaults to 0
-P, --pause <name> Pause a schedule
-p, --priority <number> Priority of each enqueued job
-q, --queue <name> Queue to put each enqueued job in, defaults to
"default"
-R, --remove <name> Remove a schedule
-r, --resume <name> Resume a paused schedule
-t, --task <name> Task name for the new schedule
-x, --lax <bool> Existing jobs each enqueued job depends on may
also have failed to allow for it to be processed
DESCRIPTION
Minion::Command::minion::schedule manages Minion schedules.
ATTRIBUTES
Minion::Command::minion::schedule inherits all attributes from Mojolicious::Command and implements the following new ones.
description
my $description = $schedule->description;
$schedule = $schedule->description('Foo');
Short description of this command, used for the command list.
usage
my $usage = $schedule->usage;
$schedule = $schedule->usage('Foo');
Usage information for this command, used for the help screen.
METHODS
Minion::Command::minion::schedule inherits all methods from Mojolicious::Command and implements the following new ones.
run
$schedule->run(@ARGV);
Run this command.
SEE ALSO
Minion, Minion::Guide, https://minion.pm, Mojolicious::Guides, https://mojolicious.org.